unit ICSoundPopupWhats;

(*	This file is part of the Internet Configuration system and is placed in the public domain for the benefit of all.

	This module implements the Sound Popup What, which drives a popup menu
	for choosing sounds.
*)

interface

	uses
		ICWindowGlobals;

	(* ***** Standard What Interface ***** *)
	
	(* See the comments in ICWindowGlobals. *)

	function WhatOpenSoundPopup (wt: WindowType; item: integer): OSStatus;
	function WhatFlushSoundPopup (wt: WindowType; item: integer): OSStatus;
	function WhatClickSoundPopup (wt: WindowType; item: integer; event: EventRecord): OSStatus;

implementation

	uses
		TextUtils,
		Sound,
		Resources,
		
		InternetConfig,

		ICMiscSubs,
		ICCommonSubs,
		ICDialogs,
		ICWindowGlobals,
		ICGlobals;

	(* ***** Utilities ***** *)

	function GetNameOfCurrentSound(wt: WindowType; item: integer) : Str255;
		(* This routine returns the text name of the currently selected
			sound.
		*)
		var
			window : WindowPtr;
			currentPopupValue : integer;
			currentSoundName : Str255;
	begin
		window := gWindows[wt].window;
		currentPopupValue := GetDControlValue(window, item);
		GetMenuItemText(GetDPopupMenuHandle(window, item), currentPopupValue, currentSoundName);
		GetNameOfCurrentSound := currentSoundName;
	end; (* GetNameOfCurrentSound *)

	(* ***** Standard What Interface ***** *)

	(* ***** What Open ***** *)
	
	function WhatOpenSoundPopup (wt: WindowType; item: integer): OSStatus;
		(* See the comments in ICWindowGlobals. *)
		var
			soundMenuH : MenuHandle;
			currentSoundName : Str255;
			attr : ICAttr;
			soundItemIndex : integer;
	begin
		soundMenuH := GetDPopupMenuHandle(gWindows[wt].window, item);
		
		(* Get the current value.  If we can't get it, use the first item from the
			menu as a default.
		*)
		if ICGetPrefStr (WhatGetDocumentInstance, gWindows[wt].items[item]^.whatKey, attr, currentSoundName) <> noErr then begin
			GetMenuItemText(soundMenuH, 1, currentSoundName);
		end; (* if *)
		WhatProcessAttributes(wt, item, attr);

		(* Find the current value in the menu. If it's not there, add it at the end. *)
		if not FindMenuItem(soundMenuH, currentSoundName, soundItemIndex) then begin
			AppendMenu(soundMenuH, '(-');
			SafeAppendMenu(soundMenuH, currentSoundName);
			soundItemIndex := CountMItems(soundMenuH);
			SetDControlMaximum(gWindows[wt].window, item, soundItemIndex);
		end; (* if *)

		(* Finish up. *)
		SetDControlValue(gWindows[wt].window, item, soundItemIndex);
		SetDControlEnable(gWindows[wt].window, item, not WhatIsLocked(wt, item));
		gWindows[wt].items[item]^.whatModified := false;
		WhatOpenSoundPopup := noErr;
	end; (* WhatOpenSoundPopup *)
	
	(* ***** What Flush ***** *)

	function WhatFlushSoundPopup (wt: WindowType; item: integer): OSStatus;
		(* See the comments in ICWindowGlobals. *)
		var
			err : OSStatus;
			currentSoundName : Str255;
	begin
		err := noErr;
		currentSoundName := GetNameOfCurrentSound(wt, item);
		if gWindows[wt].items[item]^.whatModified then begin
			err := ICSetPrefStr(WhatGetDocumentInstance, gWindows[wt].items[item]^.whatKey, ICattr_no_change, currentSoundName);
			gWindows[wt].items[item]^.whatModified := false;
		end; (* if *)
		WhatFlushSoundPopup := err;
	end; (* WhatFlushSoundPopup *)
	
	(* ***** What Click ***** *)

	function WhatClickSoundPopup (wt: WindowType; item: integer; event: EventRecord): OSStatus;
		(* See the comments in ICWindowGlobals. *)
		var
			err : OSStatus;
			currentSound : SndListHandle;
			junkLong : longint;
			junk : OSStatus;
	begin
		{$unused event}
		(* If the Option key isn't down, play the newly selected sound. *)
		err := noErr;
		if not IsKeyDown (kOptionKeyCode) then begin
			currentSound := SndListHandle(GetNamedResource('snd ', GetNameOfCurrentSound(wt, item)));
			if currentSound <> nil then begin
				junk := SndPlay(nil, currentSound, false);
			end else begin
				FlashMenuBar(0);
				Delay(6, junkLong);
				FlashMenuBar(0);
			end; (* if *)
		end; (* if *)

		(* Finish up. *)
		gWindows[wt].items[item]^.whatModified := true;
		WhatDirtyDocument;
		WhatClickSoundPopup := err;
	end; (* WhatClickSoundPopup *)
	
end. (* ICSoundPopupWhats *)